-- voronoi diagram in regular grid
-- MEDIUM!
-- 1. create a grid of some seed_window size
-- 2. randomize x,y position inside grid cell and get r,g,b of image
-- 3. calculate voronoi diagram and draw resultant image 
-- author: Sinia Petri http://www.sigmapi-design.com, 2008-2014


-- init images/process objects 
imgA = TLuaImageWrap(inputImage)
imgB = TLuaImageWrap(outputImage)
proC = TLuaProcessWrap(procObj)


r_seed = {}
g_seed = {}
b_seed = {}

x_seed =  {}
y_seed = {}

-- input variables

seed_window = 10
seed_dither = 10
-- seed dither must be less or equal seed_window

-- generate random points inside image and get pixels rgb

function generate_seeds()
   local x, y, ii, jj, iWidthM1, iHeightM1
   iWidthM1, iHeightM1 = imgA:GetImageSize()
   iWidthM1 = iWidthM1 - 1
   iHeightM1 = iHeightM1 - 1
  
   math.randomseed( os.time() )
   ii = 0
   for y = 0, iHeightM1, seed_window  do
       ii = ii + 1
       x_seed[ii] = {}      
       y_seed[ii] = {}		
       r_seed[ii] = {}      
       g_seed[ii] = {}		
       b_seed[ii] = {}      
       jj = 0	 	
       for x = 0, iWidthM1, seed_window do
	   jj = jj + 1
           y_seed[ii][jj] = y + math.random(seed_dither) - 1
           x_seed[ii][jj] = x + math.random(seed_dither) - 1
           if y_seed[ii][jj] > iHeightM1 then
              y_seed[ii][jj] = iHeightM1
	   end
	   if x_seed[ii][jj] > iWidthM1 then
              x_seed[ii][jj] = iWidthM1
	   end
           b_seed[ii][jj], g_seed[ii][jj], r_seed[ii][jj] = imgA:GetImageBGR(x_seed[ii][jj], y_seed[ii][jj])    
       end 
   end 
   return ii, jj
end

-- find closest seed
function find_closest(x, y)
    local dist, min_dist, iis, jjs
    local row, col 
    min_dist = 9999.0
    row = math.floor(y / seed_window) + 1
    col = math.floor(x / seed_window) + 1
    if row >= max_i then
       row = max_i - 1
    end
    if col >= max_j then
       col = max_j - 1
    end    
    if row <= 1 then
       row = 2
    end
    if col <= 1 then
       col = 2
    end    
    iis = row
    jjs = col
    for ii = row-1, row + 1 do
        for jj = col-1,col + 1 do
           dist = math.sqrt((x-x_seed[ii][jj])^2+(y-y_seed[ii][jj])^2)
           if dist < min_dist then
              min_dist = dist
 	      iis = ii
              jjs = jj 
           end
	end  
    end
    return r_seed[iis][jjs], g_seed[iis][jjs], b_seed[iis][jjs]            		
end

-- and now the main routine

eos = false
x = 0            
y = 0
iWidth, iHeight = imgA:GetImageSize()
total = iHeight   
done = 0   
max_i, max_j = generate_seeds()
while (not eos) do
   r,g,b = find_closest(x,y)    
   imgB:SetScanBGR(b, g, r)
   eos, x, y = imgB:ScanNext()
   if (y > done) then
      proC:ShowProgress(y, total)
      done = y
   end
end 



